@@ -144,7 +144,6 @@ static void initAP(...)
     apic->splHi();
     apic->setTimer(32, 1000);
     Core* core = new Core(sched);
-    apic->started();
     core->start();
     // NOT REACHED HERE
 }
@@ -200,20 +199,15 @@ int esInit(IInterface** nameSpace)
         apic->busFreq();
         Core::pic = apic;
 
+        Core::registerExceptionHandler(32, apic);
         if (2 <= mps->getProcessorCount())
         {
             // Startup APs
             u32 hltAP = 0x30000 + *(u16*) (0x30000 + 138);
             u32 startAP = 0x30000 + *(u16*) (0x30000 + 126);
-            *(u32*) (0x30000 + 132) = (u32) initAP;
-
-            esReport("Startap: %x\n", startAP);
-            esReport("Halt: %x\n", hltAP);
-
+            *(volatile u32*) (0x30000 + 132) = (u32) initAP;
             apic->startupAllAP(hltAP, startAP);
         }
-
-        Core::registerExceptionHandler(32, apic);
         apic->setTimer(32, 1000);
     }
 
@@ -339,6 +333,19 @@ int esInit(IInterface** nameSpace)
         breakpoint();
         //  Core::pic->startup(4);
     }
+    else if (apic)
+    {
+        // Check Architectural Performance Monitoring leaf.
+        int eax, ebx, ecx, edx;
+        Core::cpuid(0x0a, &eax, &ebx, &ecx, &edx);
+        if (0 < (eax & 0x0f) && // Check the version identifier
+            !(ebx & 0x04))      // Check the availability of UnHalted Reference Cycles event
+        {
+            esReport("Enabled NMI kernel watchdog.\n");
+
+            apic->enableWatchdog();
+        }
+    }
 
     root->bind("device/beep", static_cast<IBeep*>(pit));
 
@@ -486,6 +493,15 @@ IThread* esCreateThread(void* (*start)(void* param), void* param)
     return new Thread(start, param, IThread::Normal);
 }
 
+bool nmiHandler()
+{
+    if (apic)
+    {
+        apic->enableWatchdog();
+    }
+    return true;
+}
+
 /* write a single character      */
 void putDebugChar(int ch)
 {
