@@ -20,9 +20,6 @@
 
 // #define VERBOSE
 
-#define MIN(a, b)   (((a) <= (b)) ? (a) : (b))
-#define MAX(a, b)   (((b) <= (a)) ? (a) : (b))
-
 u32 Vesa::data[32] =
 {
     0xc0000000, // 1100000000000000
@@ -119,6 +116,8 @@ Vesa(u8* vbeInfoBlock, u8* modeInfoBlock, u8* font, IContext* device) :
 int Vesa::
 show()
 {
+    Monitor::Synchronized method(monitor);
+
     int show = count.increment();
     if (show == 1)
     {
@@ -131,6 +130,8 @@ show()
 int Vesa::
 hide()
 {
+    Monitor::Synchronized method(monitor);
+
     int show = count.decrement();
     if (show == 0)
     {
@@ -148,6 +149,8 @@ move(int dx, int dy)
 void Vesa::
 getPosition(int& x, int& y)
 {
+    Monitor::Synchronized method(monitor);
+
     x = xPosition;
     y = yPosition;
 }
@@ -155,6 +158,8 @@ getPosition(int& x, int& y)
 void Vesa::
 setPosition(int x, int y)
 {
+    Monitor::Synchronized method(monitor);
+
     if (x == xPosition && y == yPosition)
     {
         return;
@@ -195,6 +200,8 @@ setPosition(int x, int y)
 void Vesa::
 setPattern(const u32 data[32], const u32 mask[32], u16 xHotSpot, u16 yHotSpot)
 {
+    Monitor::Synchronized method(monitor);
+
     if (0 < count)
     {
         restoreBackground();
@@ -205,6 +212,7 @@ setPattern(const u32 data[32], const u32 mask[32], u16 xHotSpot, u16 yHotSpot)
     this->yHotSpot = yHotSpot;
     if (0 < count)
     {
+        saveBackground();
         drawCursor();
     }
 }
@@ -292,7 +300,7 @@ drawCursor()
     }
     if (x < 0)
     {
-        offset = x;
+        offset = -x;
         len -= -x;
         x = 0;
     }
