@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006
+ * Copyright (c) 2006, 2007
  * Nintendo Co., Ltd.
  *
  * Permission to use, copy, modify, distribute and sell this software
@@ -16,6 +16,7 @@
 #include "core.h"
 #include "elfFile.h"
 #include "process.h"
+#include "interfaceStore.h"
 
 extern ICurrentProcess* esCurrentProcess();
 
@@ -525,10 +526,6 @@ Process() :
 
     syscallTable[0].set(esCurrentProcess(), IID_ICurrentProcess);
 
-    const unsigned stackSize = 2*1024*1024;
-    Thread* thread(createThread(stackSize));
-    ASSERT(thread);
-
     Process* current(Process::getCurrentProcess());
     if (current)
     {
@@ -542,6 +539,10 @@ Process() :
 Process::
 ~Process()
 {
+#ifdef VERBOSE
+    esReport("Process::~Process %p\n", this);
+#endif
+
     setIn(0);
     setOut(0);
     setError(0);
@@ -553,7 +554,15 @@ Process::
     {
         if (1 < proxy->addRef())
         {
-            static_cast<IInterface*>(proxy->getObject())->release();
+            IInterface* object = static_cast<IInterface*>(proxy->getObject());
+            if (object)
+            {
+#ifdef VERBOSE
+                Reflect::Interface interface = getInterface(proxy->iid);   // XXX Should cache the result.
+                esReport("%s %s %p\n", __func__, interface.getName(), object);
+#endif
+                object->release();
+            }
         }
     }
 
@@ -577,7 +586,9 @@ load()
 {
     unsigned x = Core::splHi();
     Core* core = Core::getCurrentCore();
+    ASSERT(core);
     core->currentProc = this;
+    ASSERT(mmu);
     mmu->load();
     Core::splX(x);
 }
@@ -763,6 +774,11 @@ start(IFile* file, const char* argument)
 
     // XXX Check no elf file is set yet.
 
+    const unsigned stackSize = 2*1024*1024;
+    Thread* thread(createThread(stackSize));
+    ASSERT(thread);
+    syscallTable[1].set(thread, IID_IThread);   // just for reference counting
+
     Elf elf(file);
 
     if (elf.getType() != ET_EXEC)
@@ -844,13 +860,15 @@ start(IFile* file, const char* argument)
         }
     }
 
+    char name[32];
+    file->getName(name, sizeof name);
+    esReport("start %p %s\n", this, name);
+
 #ifdef VERBOSE
     dump();
     esReport("break: %p\n", end);
 #endif // VERBOSE
 
-    Thread* thread(threadList.getFirst());
-
     // Copy-in the argument.
     Ureg* ureg(static_cast<Ureg*>(thread->param));
     char* args = (char*) &ureg[1];
@@ -897,7 +915,8 @@ exit(int status)
 
     exitValue = status;
 
-    // Cancel all the threads
+    // Cancel all the threads. Note do not release threads here as they are
+    // stored in syscallTable.
     Thread* thread;
     List<Thread, &Thread::linkProcess>::Iterator iter = threadList.begin();
     while ((thread = iter.next()))
@@ -909,12 +928,9 @@ exit(int status)
         thread->setCancelState(ICurrentThread::CANCEL_ENABLE);
         thread->setCancelType(ICurrentThread::CANCEL_DEFERRED);
         thread->cancel();
-        thread->release();
     }
 
     thread = Thread::getCurrentThread();
-    thread->release();
-
     thread->exit(0);
     // NOT REACHED HERE
 
