@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2006
+ * Copyright (c) 2006, 2007
  * Nintendo Co., Ltd.
  *
  * Permission to use, copy, modify, distribute and sell this software
@@ -79,6 +79,7 @@ lock()
         {
             esThrow(err);
         }
+        addRef();
     }
     current->state = IThread::RUNNABLE;
 }
@@ -90,6 +91,7 @@ tryLock()
     switch (err)
     {
       case 0:
+        addRef();
         return true;
       case EBUSY:
       case EAGAIN:
@@ -106,6 +108,7 @@ unlock()
     {
         esThrow(err);
     }
+    release();
 }
 
 bool Monitor::
@@ -129,12 +132,22 @@ wait(s64 timeout)
     struct timespec ts;
     Thread* current(Thread::getCurrentThread());
 
-    clock_gettime(CLOCK_REALTIME, &ts);
-    ts.tv_sec += timeout / 10000000;
-    ts.tv_nsec += (timeout % 10000000) * 100;
-    current->state = IThread::TIMED_WAITING;
-    int err = pthread_cond_timedwait(&cond, &mutex, &ts);
-    current->state = IThread::RUNNABLE;
+    int err;
+    if (0 < timeout)
+    {
+        clock_gettime(CLOCK_REALTIME, &ts);
+        ts.tv_sec += timeout / 10000000;
+        ts.tv_nsec += (timeout % 10000000) * 100;
+        current->state = IThread::TIMED_WAITING;
+        err = pthread_cond_timedwait(&cond, &mutex, &ts);
+        current->state = IThread::RUNNABLE;
+    }
+    else
+    {
+        current->state = IThread::WAITING;
+        err = pthread_cond_wait(&cond, &mutex);
+        current->state = IThread::RUNNABLE;
+    }
     switch (err)
     {
     case 0:
